/**
 * Easy Shift Register Serial
 * 
 * EXEMPLO2: CONTROLE DE LEDS VIA SOFTWARE
 * 
 * Autora: Ana Paula Messina - anapaula@tecdicas.com
 * 
 * https://www.tecdicas.com/
 * 
 * 22/01/2019
 * 
 * MIT
 */

/**
 * Constantes para GPIOs do NodeMCU ESP-12E
 */
#define DATA  12 // D6
#define LATCH 13 // D7
#define CLK   15 // D8

/**
 * Constantes para GPIOs do Arduino Atmega328p
 */
//#define DATA  8
//#define LATCH 9
//#define CLK   10

int posicao = 0;
int estado = 0;
byte valorByte = 0x00;

/**
 * Monta e limpa um byte para ser manipulado no Shift Register.
 * 
 * @param byte _valorByte - Valor do byte de até 0xFF/255/8 bits.
 * @param int _posicao    - Posição do bit de 0 a 7.
 * @param int _estado     - Estado de HIGH/1 ou LOW/0 para o bit.
 */
void montarByte(byte _valorByte, int _posicao, int _estado)
{
  valorByte = _valorByte;
  posicao = _posicao;
  estado = _estado;

  //Serial.println(valorByte, BIN);
  
  if(estado == HIGH)
  {
    bitSet(valorByte, posicao);
  }
  else if(estado == LOW)
  {
    bitClear(valorByte, posicao);
  }
  
  shiftR();
  //Serial.println(valorByte, BIN);
}

/*
 * Realiza as funções básicas do Shift Register
 */
void shiftR()
{
  digitalWrite(LATCH, LOW);
  shiftOut(DATA, CLK, LSBFIRST, valorByte); 
  digitalWrite(LATCH, HIGH);
}

/**
 * Funções para cada led.
 * É repetitivo mesmo.
 */
void led0()
{ 
  posicao = 0;
  int bRead = bitRead(valorByte, posicao);
  if(bRead == HIGH)
  {
    montarByte(valorByte, posicao, LOW);
    Serial.println(bitRead(valorByte, posicao));
  }
  else
  {
    montarByte(valorByte, posicao, HIGH);
    Serial.println(bitRead(valorByte, posicao));
  }
}

void led1()
{ 
  posicao = 1;
  int bRead = bitRead(valorByte, posicao);
  if(bRead == HIGH)
  {
    montarByte(valorByte, posicao, LOW);
    Serial.println(bitRead(valorByte, posicao));
  }
  else
  {
    montarByte(valorByte, posicao, HIGH);
    Serial.println(bitRead(valorByte, posicao));
  }
}

void led2()
{ 
  posicao = 2;
  int bRead = bitRead(valorByte, posicao);
  if(bRead == HIGH)
  {
    montarByte(valorByte, posicao, LOW);
    Serial.println(bitRead(valorByte, posicao));
  }
  else
  {
    montarByte(valorByte, posicao, HIGH);
    Serial.println(bitRead(valorByte, posicao));
  }
}
void led3()
{ 
  posicao = 3;
  int bRead = bitRead(valorByte, posicao);
  if(bRead == HIGH)
  {
    montarByte(valorByte, posicao, LOW);
    Serial.println(bitRead(valorByte, posicao));
  }
  else
  {
    montarByte(valorByte, posicao, HIGH);
    Serial.println(bitRead(valorByte, posicao));
  }
}

void led4()
{ 
  posicao = 4;
  int bRead = bitRead(valorByte, posicao);
  if(bRead == HIGH)
  {
    montarByte(valorByte, posicao, LOW);
    Serial.println(bitRead(valorByte, posicao));
  }
  else
  {
    montarByte(valorByte, posicao, HIGH);
    Serial.println(bitRead(valorByte, posicao));
  }
}

void led5()
{ 
  posicao = 5;
  int bRead = bitRead(valorByte, posicao);
  if(bRead == HIGH)
  {
    montarByte(valorByte, posicao, LOW);
    Serial.println(bitRead(valorByte, posicao));
  }
  else
  {
    montarByte(valorByte, posicao, HIGH);
    Serial.println(bitRead(valorByte, posicao));
  }
}

void led6()
{ 
  posicao = 6;
  int bRead = bitRead(valorByte, posicao);
  if(bRead == HIGH)
  {
    montarByte(valorByte, posicao, LOW);
    Serial.println(bitRead(valorByte, posicao));
  }
  else
  {
    montarByte(valorByte, posicao, HIGH);
    Serial.println(bitRead(valorByte, posicao));
  }
}

void led7()
{ 
  posicao = 7;
  int bRead = bitRead(valorByte, posicao);
  if(bRead == HIGH)
  {
    montarByte(valorByte, posicao, LOW);
    Serial.println(bitRead(valorByte, posicao));
  }
  else
  {
    montarByte(valorByte, posicao, HIGH);
    Serial.println(bitRead(valorByte, posicao));
  }
}

/**
 * Realiza a comunicação serial enviando os dados requeridos.
 * Utilizar Monitor Serial, Putty, terminal Linux ou software que realize comunicação serial.
 */
void comSerial()
{
  if(Serial.available())       
  {
    char op = Serial.read();    
    
    if (op == '0')              
    {
      led0();
    }
    else if (op == '1')             
    {
      led1();
    }
    else if (op == '2')              
    {
      led2();
    }
    else if (op == '3')              
    {
      led3();
    }
    else if (op == '4')              
    {
      led4();
    }
    else if (op == '5')              
    {
      led5();
    }
    else if (op == '6')              
    {
      led6();
    }
    else if (op == '7')              
    {
      led7();
    }
    else if(op == '8')
    {
      apagaByte();
    }
  }
}

/**
 * Seta todas posições dos bits em zero.
 */
void apagaByte()
{
  for (int i=0;i<=7;i++)
  {
    montarByte(0x00, i, LOW);
  }
}

void setup()
{
  Serial.begin(9600);
  pinMode(DATA, OUTPUT);
  pinMode(LATCH, OUTPUT);
  pinMode(CLK, OUTPUT); 
  apagaByte();
}

void loop() 
{   
  comSerial(); 
}
